@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2015 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -567,7 +567,7 @@ static int firebird_handle_get_attribute(pdo_dbh_t *dbh, long attr, zval *val TS
 #else
 			HMODULE l = GetModuleHandle("fbclient");
 
-			if (!l && !(l = GetModuleHandle("gds32"))) {
+			if (!l) {
 				break;
 			}
 			info_func = (info_func_t)GetProcAddress(l, "isc_get_client_version");
@@ -575,9 +575,7 @@ static int firebird_handle_get_attribute(pdo_dbh_t *dbh, long attr, zval *val TS
 			if (info_func) {
 				info_func(tmp);
 				ZVAL_STRING(val,tmp,1);
-			} else {
-				ZVAL_STRING(val,"Firebird 1.0/Interbase 6",1);
-			}
+			} 
 #else
 			ZVAL_NULL(val);
 #endif
